// CUDlg.cpp : implementation file
//
// CrackUp version 1.0
// Copyright  1998 Ziff-Davis, Inc.
// First published in PC Magazine, US Edition, November 3, 1998
// Written by Bob Flanders and Gregory A. Wolking

#include "stdafx.h"
#include <winioctl.h>
#include "CrackUp.h"
#include "AboutDlg.h"
#include "CUDlg.h"
#include "ScanDlg.h"
#include "RptPage.h"
#include "CfgPage.h"
#include "CUSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Macro and other definitions
#define	IDT_TIMER	0xb0bf					// my timer id


//////////////////////////
// Global Variables
//////////////////////////

extern HWND				g_hMasterWindow;		// handle to main window
extern CRITICAL_SECTION	g_csBG,					// background is running
						g_csScanning;			// a scan is scan in progress
extern BOOL				g_bPopup,				// popups wanted
						g_bQuickMode,			// "quick mode" flag.
						g_bScanInProgress,		// scan is happening
						g_bBackground,			// use background thread
						g_bRunThread,			// run thread until FALSE
						g_bStopScanNow,			// quit the scan
						g_bMessageActive,		// message box flag.
						g_bNewWindows;			// true if OSR2
extern WORD				g_wMaxFrag,				// maximum frag allowed
						g_wNDrivesSelected,		// number of drives checked
						g_wScanInterval,		// hours between scans
						g_wTimeTicks,			// "ticks" left this hour.
						g_wHoursLeft;			// time left until resume
extern HANDLE			g_hMyThread;			// thread handle
extern CString			g_strIniFile;			// INI filename.
extern OSVERSIONINFO	g_stOsVer;				// version of the os
extern DRVTABENT		g_stDrvTbl[MAXDRIVES];	// drive table 
extern WORD				g_wNDrives;				// number of valid drives found
extern CDialog*			g_pRptWin;				// Pointer to report window
extern HICON			g_hIcons[4];

///////////////////////////////
// Global Function Declarations
///////////////////////////////
extern void CheckLevels();
extern void ErrorMsg(char * szMsg);
extern void ScanDrives(CScanMsg* lpSx);
extern void StartBgScan();
extern void WriteProfile(void);
extern BOOL BuildSystray(HWND hwnd, UINT uID, HICON hicon, LPSTR lpszTip);
extern BOOL DrvEligible(char* szDrv, PDRVTABENT pstDrvTab);
extern BOOL KillSystray(HWND hwnd, UINT uID);
extern float DefragsPct(PDRVTABENT pD, int iFormula);				// defrag percentage calculator


/////////////////////////////////////////////////////////////////////////////
// CCrackUpDlg dialog

CCrackUpDlg::CCrackUpDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCrackUpDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCrackUpDlg)
	//}}AFX_DATA_INIT
}

void CCrackUpDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCrackUpDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCrackUpDlg, CDialog)
	//{{AFX_MSG_MAP(CCrackUpDlg)
	ON_WM_TIMER()
	ON_COMMAND(ID_ABOUT, OnAbout)
	ON_COMMAND(ID_SCANNOW, OnScanNow)
	ON_COMMAND(ID_OPEN, OnOpen)
	ON_COMMAND(ID_EXIT, OnExit)
	//}}AFX_MSG_MAP
	ON_MESSAGE(MYWM_NOTIFYICON, OnTrayIcon)
	ON_MESSAGE(MYWM_CHECKLEVELS, OnCheckLevels)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCrackUpDlg message handlers

BOOL CCrackUpDlg::OnInitDialog()
{
	char	   *sP;										// work string pointer
	char		szVal[20];								// value response
	BOOL		bIniFound = FALSE;						// .ini file found
	CWinApp		*pApp;									// pointer to our CWinApp object.
	int			i;										// loop counter.

	pApp = AfxGetApp();
	CDialog::OnInitDialog();

	m_bDialogActive = FALSE;	// Used to signal when one or more modal dialogs is active.
	m_pActiveDialog = NULL;		// Pointer to current modal dialog.
	m_bAboutBoxActive = FALSE;
	m_pAboutBox = NULL;
	// Set application icon. (For this app, it's only needed to
	// show the proper icon in the Alt-Tab dialog used to switch between
	// applications.
	m_hIcon = pApp->LoadIcon(IDR_MAINFRAME);
	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);
	// *********************************************************************
	// Don't show disk errors to the user ..
	// *********************************************************************
	SetErrorMode(SEM_FAILCRITICALERRORS);					// return critical erros
	// *********************************************************************
	//	Check the OS Version. We must be under Windows 95. 
	// *********************************************************************
	g_stOsVer.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);		// set struct size
	GetVersionEx(&g_stOsVer);									// get the OS version
	if (g_stOsVer.dwPlatformId != VER_PLATFORM_WIN32_WINDOWS)	// q. is this Win95?
	{															// a. no .. 	
		ErrorMsg(ERMBADVERSION);								// tell the user
		ExitProcess(INITERR);									// and return to Windows
	}
	g_bNewWindows = (LOWORD(g_stOsVer.dwBuildNumber) > NEWWINBUILD);	// set new windows flag
	g_hMasterWindow = m_hWnd;										// save window handle	
	if (!g_bQuickMode)	// If not in "Quick Scan" mode
	{
		// Load the icons used for animation. NOTE: if you change these icons yourself,
		// their identifiers must be sequential!
		for (i = 0; i <= 3; i++)
			g_hIcons[i] = (HICON) LoadImage(pApp->m_hInstance, MAKEINTRESOURCE(IDR_ICONSMALL0 + i), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
		BuildSystray(m_hWnd,								// this window's handle
				 IDR_ICONSMALL0, 							// use this resource
				 g_hIcons[0],								// use this icon
				 "CrackUp");								// start with "Crackup
	}

	// *********************************************************************
	//	Build the list of drives in the system
	// *********************************************************************
	GetLogicalDriveStrings(DRVSTRLEN, m_szDrives);			// get the list of drives
	sP = m_szDrives;										// init the string pointer
	m_szGoodDrvs[0] = 0;									// init the good drives 
	InitializeCriticalSection(&g_csScanning);				// init critsec object
	InitializeCriticalSection(&g_csBG);						// background running
	GetPrivateProfileString("INIT",							// get [INIT] ..
							"CONFIGURED",					// CONFIGURED switch
							"*",							// default to star
							szVal,							// put it here
							2,								// max 2 chars
							g_strIniFile);					// from our init file
	bIniFound = (szVal[0] != '*');							// set ini file found flag
	GetPrivateProfileString("INIT",							// get [INIT] ..
							"MAXFRAGPERCENT",				// MAXFRAGPERCENT
							"20",							// default to 20 percent
							szVal,							// put it here
							10,								// max 10 chars
							g_strIniFile);					// from our init file
	g_wMaxFrag = (WORD) atoi(szVal);						// get the value
	g_wMaxFrag = (g_wMaxFrag < 1) ? (WORD) 20 : g_wMaxFrag;	// don't allow 0
	g_wMaxFrag = (g_wMaxFrag > 99) ? (WORD) 99 : g_wMaxFrag;// or over 100
	GetPrivateProfileString("INIT",							// get [INIT] ..
							"POPUP",						// POPUP
							"Y",							// default to YES
							szVal,							// put it here
							10,								// max 10 chars
							g_strIniFile);					// from our init file
	g_bPopup = ((*szVal & 0x5f) == 'Y');  					// set popup value
	GetPrivateProfileString("INIT",							// get [INIT] ..
							"BACKGROUND",					// Background process
							"Y",							// default to YES
							szVal,							// put it here
							10,								// max 10 chars
							g_strIniFile);					// from our init file
	g_bBackground = ((*szVal & 0x5f) == 'Y'); 				// set background value
	GetPrivateProfileString("INIT",							// get [INIT] ..
							"BGTIMER",						// Background timer
							"4",							// default to 4 hours
							szVal,							// put it here
							10,								// max 10 chars
							g_strIniFile);					// from our init file
	g_wScanInterval = (WORD) atoi(szVal);					// set the timer value
	if (g_wScanInterval < 1) g_wScanInterval = DFLTIME;		// to at least 1 hour
	g_wHoursLeft = g_wScanInterval;
	g_wTimeTicks = 360;
	while (*sP != NULL)										// while there are drives
	{			
		if (DrvEligible(sP, &g_stDrvTbl[g_wNDrives]))		// q. is the drive elegible?
				{											// a. yes ..
			strncat(m_szGoodDrvs, sP, 1);					// save the drive letter
			g_wNDrives++;									// increment nbr of drives
		}
		sP += strlen(sP)+ 1;								// sP -> next drive
	}
	if (g_wNDrives == 0)									// q. any drives found?
	{														// a. no ..
		ErrorMsg(ERMNODRIVES);								// tell the user
		DestroyWindow();									// and exit the program
		return TRUE;										// we're out of here
	}
	StartBgScan();											// start the background thread
	if (!g_bQuickMode)							// If not in "quick scan" mode
	{
		m_intMyTimer = SetTimer(IDT_TIMER,		// Try to start a timer
				 TENSEC,						// for ten second intervals
				 NULL);							// no callback
		if (m_intMyTimer == 0)					// If unable to set timer, display prompt and quit.
		{
			MessageBox("Could not create timer for background operation.", "CrackUp Error", MB_ICONSTOP);
			PostMessage(WM_COMMAND, ID_EXIT, 0);
			return TRUE;
		}
		if (g_bBackground && bIniFound)		// Background operation requested and drives configured?
		{
			g_wHoursLeft = 1;				// Let first scan start at next timer tick.
			g_wTimeTicks = 0;
		}
	}
	else										// Quick Scan requested, so
	{
		m_intMyTimer = 0;						// No timer in use.
	}
	if (!bIniFound)								// q. configured?
	{											// a. no ..
		PostMessage(WM_COMMAND, ID_OPEN, 0);	// Open window.
	}
	else
	{
		if (g_bBackground && !g_bQuickMode)
		{
			g_wHoursLeft = 1;
			g_wTimeTicks = 0;
		}
		else
			PostMessage(WM_COMMAND, ID_SCANNOW, 0);
	}
	return TRUE;
}

BOOL CCrackUpDlg::DestroyWindow() 
{
	WinHelp(0L, HELP_QUIT);
	if (!g_bQuickMode)						// If not in "quick" mode,
	{										// Remove tray icon.
		KillSystray(m_hWnd,					// this window's handle
					IDR_ICONSMALL0);		// use this resource
		// Note: The following isn't absolutely necessary -- DestroyWindow
		//       is _supposed_ to release any timers belonging to that window.
		//       However, better safe than sorry!
		if (m_intMyTimer != 0)				// If it was created,
			KillTimer(m_intMyTimer);		// release the timer.
		for (int i = 0; i <= 3; i++)
		{
			if (g_hIcons[i])
				DestroyIcon(g_hIcons[i]);
		}
	}
	//Note: Since this is the app's main window, CDialog::DestroyWindow() takes
	//      care of posting a WM_QUIT message to the app's message queue.
	return CDialog::DestroyWindow();
}

// Message handlers

// Called when our timer fires.
// Note: This app only uses one timer, so we don't need the nIDevent parameter.
// It is commented out here to suppress an "unused formal parameter" warning from the compiler.

void CCrackUpDlg::OnTimer(UINT/* nIDEvent*/) 
{
	if ((--g_wTimeTicks <= 0) || (g_wTimeTicks > 360))	// q. "ticks" out of range?
	{													// a. yes ..
		g_wTimeTicks = 360;								// reset "ticks" left in this hour
		g_wHoursLeft--;									// and assume an hour has passed
	}
	if (g_pRptWin != NULL)
		g_pRptWin->PostMessage(MYWM_UPDATETIMELEFT, 0, 0);
	if (g_wHoursLeft)									// q. timeout expired?
		return;											// a. no .. exit now
	g_wHoursLeft = g_wScanInterval;						// Reset timer.
	if (g_bBackground && !g_bScanInProgress &&			// q. background enabled
		g_wNDrivesSelected > 0)							//    and at least one drive selected?
		ResumeThread(g_hMyThread);						// yes .. resume the thread.
}

// Called when user manipulates our tray icon.
void CCrackUpDlg::OnTrayIcon(WPARAM, LPARAM lParam)
{
	POINT		stP;		// point for pos
	CMenu		cMnu;		// menu object
	CMenu*		cSub;
	HWND		hTarget;	// Handle used to search for our message boxes.			

	// If message is not a mouse move and either one of our dialogs or
	// a message box is active,
	// If a dialog or message box is active
	if  (m_bDialogActive || m_bAboutBoxActive || g_bMessageActive || g_pRptWin != NULL)
	{
		if (lParam == WM_LBUTTONDBLCLK || lParam == WM_RBUTTONUP)
		{
			if (g_bMessageActive)		// Is a message box active?
			// Unfortunately, we don't know the handle of the message box window
			// that MessageBox() creates, so we have to look for it.
			{
				// Search for "no drives selected" error message.
				hTarget = FindWindowEx(NULL, NULL, "#32770", "CrackUp Error");
				// If not found,
				if (hTarget == NULL)
					// Search for other message boxes.
					hTarget = FindWindowEx(NULL, NULL, "#32770", "CrackUp Notice");
				if (hTarget)							// If message box was found,
					::SetForegroundWindow(hTarget);		// bring it to the foreground.
			}
			// Not a message box, must be a dialog.
			else if (m_bAboutBoxActive)					// "About" box?
			{
				m_pAboutBox->SetForegroundWindow();		// Activate it if so.
			}
			else
			{
				if (m_pActiveDialog != NULL)					// Is it some other dialog?
					m_pActiveDialog->SetForegroundWindow();		// Activate it if so.
				else											// Otherwise,
					if (g_pRptWin != NULL)						// is it the Property Sheet?
						g_pRptWin->SetForegroundWindow();		// Activate it if so.
			}
		}
	}
	else
	{
		switch (lParam)
		{
		case WM_RBUTTONUP:			// Right click? 
			GetCursorPos(&stP);							// get current position
			cMnu.LoadMenu(IDR_MENU1);					// and load the menu
			cSub = cMnu.GetSubMenu(0);					// get the sub menu
			SetForegroundWindow();						// ensure we're in front
			cSub->EnableMenuItem(ID_SCANNOW, (g_wNDrivesSelected == 0) ? MF_GRAYED : MF_ENABLED);
			cSub->TrackPopupMenu(TPM_CENTERALIGN		// Build popup with left aligned
									| TPM_RIGHTBUTTON,	// and right button active
									stP.x,				// at these
									stP.y,				// coordinates
									this,				// under the dialog box
									NULL);				// and kill menu if no click
			PostMessage(WM_NULL, 0 ,0);				// ensure menu disappears
			cMnu.DestroyMenu();						// kill the menu
			break;
		case WM_LBUTTONDBLCLK:		// Left double-click?
			PostMessage(WM_COMMAND, ID_OPEN, 0);	// Open the Property Sheet.
		}
	}
}

// Handles message posted by background thread when a scan is completed.
void CCrackUpDlg::OnCheckLevels(WPARAM wParam, LPARAM)
{
	if (wParam != IDM_CHKFRAG)					// q. checkfrag request?
		return;									// a. no .. we don't care
	if (g_pRptWin != NULL)						// If dialog is open,
		g_pRptWin->PostMessage(MYWM_SCANCOMPLETE, 0, 0);	// tell it to update itself.
	CheckLevels();								// run a check levels request
}

// Menu Command handlers

// "About"
void CCrackUpDlg::OnAbout()
{
	m_bAboutBoxActive = TRUE;
	CAboutDlg dlgAbout;						// create about dialog
	m_pAboutBox = (CDialog*) &dlgAbout;		// save pointer to dialog for tray icon handler
	dlgAbout.DoModal();						// and display it 
	m_bAboutBoxActive = FALSE;
	m_pAboutBox = NULL;
}

// "Scan Now"
void CCrackUpDlg::OnScanNow()
{
	// If the About box is active,
	if (m_bAboutBoxActive)
	{
		// Make sure it is closed before we proceed.
		m_pAboutBox->SendMessage(WM_COMMAND, IDOK, 0);
	}
	CScanMsg dlgSm;							// build the dialog box

	// To be sure that a background scan doesn't start shortly
	// after a foreground scan does, reset the clock.
	g_wHoursLeft = g_wScanInterval;			
	g_wTimeTicks = 360;
	m_bDialogActive = TRUE;					// set flag for OnTrayIcon()
	dlgSm.Create(IDD_SCANNING, this);		// create the window
	m_pActiveDialog = (CDialog*) &dlgSm;	// save pointer for OnTrayIcon()
	g_bStopScanNow = TRUE;					// signal background scan to stop,
	EnterCriticalSection(&g_csScanning);	// and make sure it has before we proceed.
	LeaveCriticalSection(&g_csScanning);
	ScanDrives(&dlgSm);						// and do a scan
	dlgSm.DestroyWindow();					// kill the message dialog
	m_bDialogActive = FALSE;				// clear flag
	m_pActiveDialog = NULL;					// NULL the pointer.
	OnOpen();								// open the Report Page.
}

// "Open"
void CCrackUpDlg::OnOpen()
{
	BOOL bDriveChecked = FALSE;
	int iResult;
	int iIndex = 0;		// Default page index for property sheet = Report page.

	if (g_pRptWin != NULL)					// q. PropertySheet already open?
	{
		if (IsWindow(g_pRptWin->m_hWnd))
			g_pRptWin->SetForegroundWindow();	// a. Yes -- bring it to foreground
		return;									// and we're done.
	}
	for (int i = 0; i < g_wNDrives; i++)		// loop to check for checked drives
	{
		if (g_stDrvTbl[i].bCheckDrive)			// q. check this drive?
		{										// a. yes ..
			bDriveChecked = TRUE;				// show one is checked
			break;								// and leave the loop
		}
	}
	if (!bDriveChecked)							// q. any checked?
	{											// a. no .. 
		g_bMessageActive = TRUE;
		iResult = ::MessageBox(m_hWnd, "There are no "	// tell the user the problem
				 "drives currently selected.\n\n"		// and ask if they want to see
				 "Do you want to open the "				// if they answer yes
				 "Configure Page now?",					// we'll send a message 
				 "CrackUp Error",						// to open the configuration
				 MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND);		// dialog box.
		g_bMessageActive = FALSE;
		if (iResult == IDYES)		// If user said "Yes",
			iIndex = 1;				// set index for configure page.
		else						// Otherwise,
			return;					// nothing to do.
	}
	m_bDialogActive = TRUE;					// Set flag for tray icon handler.
	CCUSheet cD("CrackUp", this, iIndex);	// Create our property sheet.
	m_pActiveDialog = (CDialog*) &cD;		// Save pointer for tray icon handler.
	g_pRptWin = m_pActiveDialog;			// Save pointer for background thread.
	cD.DoModal();							// Activate the sheet.
	m_bDialogActive = FALSE;				// Clear flags and pointers.
	m_pActiveDialog = NULL;					// Note: CCUSheet's destructor takes care of NULLing g_pRptWin 
}

// "Exit"
void CCrackUpDlg::OnExit()
{
	g_bStopScanNow = TRUE;					// stop any scan right now
	g_bRunThread = FALSE;					// tell thread to stop
	ResumeThread(g_hMyThread);				// resume the thread
	EnterCriticalSection(&g_csBG);			// wait for BG to end
	LeaveCriticalSection(&g_csBG);
	CloseHandle(g_hMyThread);				// release thread handle.
	DeleteCriticalSection(&g_csScanning);	// release critsec object
	DeleteCriticalSection(&g_csBG);
	DestroyWindow();						// end program.
}

